# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.protocol.base import BaseProtocol
from driverng.transport.base import TransportException
from driverng.utils import ACK, ENQ, EOT, ETX, STX, byte2log


class MaglumiAstmProtocol(BaseProtocol):
    def __init__(
        self,
        mode_bloc=False,
        max_car_bloc=240,
        end_payload_with_cr=True,
        mode_iq200=False,
        mode_etb=False,
        checksum_ignore_end=False,
        **kwargs
    ):
        """:param mode_bloc: True pour forcer l'envoi en mode bloc
        :param max_car_bloc: Longueur max du payload en mode bloc
        """
        super(MaglumiAstmProtocol, self).__init__(**kwargs)
        self.mode_bloc = mode_bloc
        self.max_car_bloc = max_car_bloc
        self.end_payload_with_cr = end_payload_with_cr
        self.mode_etb = mode_etb
        self.mode_iq200 = mode_iq200

        # Compter le dernier caractère (etx ou etb) dans le checksum ?
        self.checksum_ignore_end = checksum_ignore_end

    def send_trames(self, trames):
        """Envoie une liste de trame (message) sur la connexion.
        Les trames sont construites avec `build_trames` puis envoyés
        suivant le protocole automate::

            -> <enq>
            <- <ack>
            -> DATA
            <- <ack>
            -> <eot>

        :return: False si erreur sinon True (A voir: exceptions)
        :param trames: liste de trames à envoyer
        """

        self.write(ENQ)
        try:
            data = self.read(timeout=15)
        except TransportException:
            self.logger.info("Aucun acquittement dans le timeout de 15 secondes")
            return False
        if not data:
            self.write(EOT)
            return False

        if data[0] == ENQ:
            # L'automate a la priorité, le driver attends un temps
            # aléatoire entre 10 et 20 secondes
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "ENQ clash: wait %ds before re-sending <enq>", self.rtimeout
            )
            return False

        if data[0] != ACK:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<ack> expected in %s: wait %ds before re-sending <enq>",
                byte2log(data),
                self.rtimeout,
            )
            return False
        self.write(STX)
        try:
            data = self.read(timeout=15)
        except TransportException:
            self.logger.info("Aucun acquittement STX dans le timeout de 15 secondes")
            return False
        if not data or data[0] != ACK:
            self.logger.info("<ack> expected ")
            self.write(EOT)
            return False

        index = 0

        while True:
            try:
                trame = trames[index]
            except IndexError:
                break
            index += 1

            self.write(trame)

        data = self.read(timeout=15)

        if not data:
            self.logger.info("send trames wait ack KO ")
            self.write(EOT)
            return False

        if data[0] not in (ACK, EOT):
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<ack> or <eot> expected in %s: wait %ds before re-sending <enq>",
                byte2log(data),
                self.rtimeout,
            )
            self.write(EOT)
            return False

        self.write(ETX)
        data = self.read(timeout=15)
        self.write(EOT)
        data = self.read(timeout=15)

        return True

    def build_trames(self, message):
        """Rajoute des informations de transport sur un message:

            <stx><indice><payload><cr

        Le checksum se fait sur <indice><payload><cr><etx>

        :param message: Le message à envoyer
        :return: Liste de trame avec les informations de transport.

        """
        return [b"\r".join(message.splitlines()) + b"\r"]

    def recv_trames(self):
        """Reçoit une liste de trames (message) sur la connexion::

            <- <enq>
            -> <ack>
            <- <stx>
            -> <ack>
            <- DATA
            -> <ack>
            <- <etx>
            -> <ack>
            <- <eot>
            -> <ack>

        :return: False si erreur sinon la liste des messages reçus
        """

        # timeout non spécifié dans le protocole
        data = self.read(timeout=1)

        if not data or data[0] != ENQ:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<enq> expected in %s: wait %ds", byte2log(data), self.rtimeout
            )
            return False

        self.write(ACK)

        trames = b""

        # il faut faire un read avec un timeout 15
        # pour l'ensemble de la trame
        ctrl = self.read(timeout=15)
        if not ctrl:
            return False
        if ctrl[0] != STX:
            self.logger.warning("<stx> expexted in %s", byte2log(ctrl))
            return False
        self.write(ACK)
        data = self.readline(break_func=lambda buf: buf[-6:] == b"L|1|N\r", timeout=15)
        self.write(ACK)
        trames += data
        ctrl = self.read(timeout=15)
        if ctrl[0] != ETX:
            self.logger.warning("<etx> expexted in %s", byte2log(ctrl))
        self.write(ACK)
        ctrl = self.read(timeout=15)
        if ctrl[0] != EOT:
            self.logger.warning("<eot> expexted in %s", byte2log(ctrl))
        self.write(ACK)

        messages = b"\n".join(trames.splitlines())
        return self.get_messages(messages.splitlines())

    @staticmethod
    def get_messages(trames):
        """Retourne la liste de messages '\n' joined."""
        # ASTM over ASTM
        if trames and bytearray([trames[0][0]]) == b"H":
            messages = []
            current_message = []
            for line in trames:
                if line and bytearray([line[0]]) == b"H":
                    if current_message:
                        messages.append(b"\n".join(current_message))
                        current_message = []
                current_message.append(line)
            if current_message:
                messages.append(b"\n".join(current_message))
            return messages
        else:
            return [b"\n".join(trames)]
